/* Copyright 2016 Integration New Media Inc. */

package com.fadel.aem.ARCProxy;

import com.fadel.aem.*;
import org.apache.felix.scr.annotations.Component;
import org.apache.felix.scr.annotations.Activate;
import org.apache.felix.scr.annotations.Service;
import java.util.Dictionary;
import org.apache.felix.scr.annotations.Property;
import org.apache.sling.commons.osgi.PropertiesUtil;
import org.osgi.service.component.ComponentContext;
import org.apache.felix.scr.annotations.*;



@Component(immediate = true, metatype = true, label = "FADEL ARC service API Config", description = "This config it's use to connect the AEM proxy with the ARC webservice")
@Service(value = ARCConfig.class)
public class ARCConfigImpl implements ARCConfig {

	/**
	 * Property that will be configured in the OSGI component and will contain
	 * the Username to connect with the ARC service.
	 */
	@Property(label = "User", description = "User for ARC conection", value = "")
	public static final String USER_ARC = "userARC";
	private String userARC;

	/**
	 * Property that will be configured in the OSGI component and will contain
	 * the Password to connect with the ARC service.
	 */
	@Property(label = "Password", description = "Passw for ARC conection", value = "")
	public static final String PASSW_ARC = "passARC";
	private String passARC;

	/**
	 * Property that will be configured in the OSGI component and will contain
	 * the URL to connect with the ARC service. Ex.
	 * http://demo.fadel.com:8082/ipmSuite5/ws/V1/
	 * 
	 * Important: You must include the /ipmSuite5/ws/V1/ segment in the address
	 * including the final "/" because the URL sent in the POST body request
	 * will be append to this one to establish the connection with the ARC
	 * RestFul web service.
	 */
	@Property(label = "ARC base URL", description = "Ex. http://demo.fadel.com:8082/ipmSuite5/ws/V1/", value = "")
	public static final String BASE_URL = "baseURL";
	private String baseURL;

	
	/**
	 * Property that will set the Asset's file name as externalID ignoring the value in the
	 * External ID Property
	 */
	@Property(label = "Filename as externalID", description = "Use Asset file's name without Ext as externalID, ignoring the External ID Property value", boolValue=false)
	public static final String FILENAMEEXT_ID = "fileNameExtID";
	private Boolean fileNameExtID;
	
	/**
	 * Property that will be configured in the OSGI component and will contain
	 * the ExternalID path for linking with ARC content.
	 */
	@Property(label = "External ID Property", description = "External ID Property", value = "dam:sha1")
	public static final String EXTERNAL_ID = "externalID";
	private String externalID;
	
	
	/**
	 * Property that will be configured in the OSGI component and will be
	 * used to enable or disable the multiple selection in the Format-Territory dialog.
	 */
	@Property(label = "Selection type", description = "Change the selection type in the Territories and Formats dialog",  
			options = {
			        @PropertyOption(name = "Multiple", value = "Multiple"),
			        @PropertyOption(name = "Single",   value = "Single")
			    }
			)
	public static final String SELECTION_Type = "selectionType";
	private String selectionType;
	
	
	/**
	 * Component context for further properties extraction
	 */
	private ComponentContext context;
	
	
	/**
	 * Activate method for ComponentContext retrieval
	 * @param componentContext
	 */
	@Activate
	protected void activate(ComponentContext componentContext) {
		context = componentContext;
		configure(context.getProperties());
	}
	

	
	/**
	 * Getters section
	 */
	@Override
	public String getUserARC() {
		return userARC;
	}
	
	@Override
	public String getPassARC() {
		return passARC;
	}
	
	@Override
	public String getBaseURL() {
		return baseURL;
	}
	
	@Override
	public String getExternalID() {
		return externalID;
	}
	
	@Override
	public String getSelectionType() {
		return selectionType;
	}
	
	@Override
	public Boolean getFileNameExtID() {
		return fileNameExtID;
	}
	
	
	
	
	/**
	 * Utility method used for reading the 3 config parameters set in the OSGI
	 * config console. This method will be called in each POST processing
	 * request.
	 * 
	 * @param properties
	 *            Properties dictionary from the component context object
	 */
	protected void configure(Dictionary<?, ?> properties) {
		this.userARC = PropertiesUtil.toString(properties.get(USER_ARC), null);
		this.passARC = PropertiesUtil.toString(properties.get(PASSW_ARC), null);
		this.baseURL = PropertiesUtil.toString(properties.get(BASE_URL), null);
		this.externalID = PropertiesUtil.toString(properties.get(EXTERNAL_ID), null);
		this.selectionType = PropertiesUtil.toString(properties.get(SELECTION_Type), null);
		this.fileNameExtID = PropertiesUtil.toBoolean(properties.get(FILENAMEEXT_ID), false);		
	}
	
	
}
