/* Copyright 2016 Integration New Media Inc. */

package com.fadel.aem.ARCProxy;

import java.io.IOException;
import org.apache.felix.scr.annotations.sling.SlingServlet;
import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.SlingHttpServletResponse;
import org.apache.sling.api.servlets.SlingAllMethodsServlet;
import org.apache.sling.commons.json.JSONObject;
import org.slf4j.LoggerFactory;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.List;
import java.util.Scanner;
import java.rmi.ServerException;
import java.io.OutputStream;
import org.slf4j.Logger;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.io.IOUtils;
import org.apache.felix.scr.annotations.*;

/**
 * A SlingServlet who will act as a Proxy for handling the ARC calling. 
 * It have 3 properties values which can be set in /system/console/configMgr.
 */

/**
 * The SlingServlet will be addressed by /bin/assetClearance in the AEM instance
 * and it will process only POST request.
 */
@SlingServlet(paths = "/bin/assetClearance", methods = { "POST" })
public class FadelServlet extends SlingAllMethodsServlet {


	/**
	 * OSGI service reference for OSGI config retrieval 
	 */
	@Reference
	ARCConfig config;
	
	
	private static final long serialVersionUID = -3960692666512058118L;

	/**
	 * Just a logger for the web service.
	 */
	private static final Logger LOGGER = LoggerFactory.getLogger(FadelServlet.class);

	
	/**
	 * Method for processing all the POST request made to this Servlet.
	 * 
	 * Important: All the request to this servlet will be POST, but this method
	 * will do GET and POST calls to the ARC service, depending of the body
	 * parameter "method"
	 */
	@Override
	protected void doPost(SlingHttpServletRequest request, SlingHttpServletResponse response)
			throws ServerException, IOException {
		
		int responseCode = 0;
		
		try {

			// Get the submitted form data that is sent from the
			// CQ web page
			

			String body = extractPostRequestBody(request);
			JSONObject obj = new JSONObject(body);
			String urlMethod = obj.getString("method");
			String requestProperty = obj.getString("contentType");
			String data = obj.getString("data");
			String credentials = config.getUserARC() + ':' + config.getPassARC();

			byte[] bytesEncoded = Base64.encodeBase64(credentials.getBytes());
			String encode = new String(bytesEncoded, "UTF-8");

			String authorization = "Basic " + encode;
			String address = config.getBaseURL() + obj.getString("uri"); // this
			String message = "";
			BufferedReader br;
			List<String> results;

			URL url = new URL(address);

			HttpURLConnection conn = (HttpURLConnection) url.openConnection();

			conn.setRequestMethod(urlMethod);
			conn.setRequestProperty("Content-Type", requestProperty);
			conn.setRequestProperty("Authorization", authorization);

			if (urlMethod.equalsIgnoreCase("POST")) {
				conn.setDoOutput(true);
				conn.setRequestProperty("Accept", "application/json");

				if ((data != null) && !(data.isEmpty()) && data.length() > 2) {
					OutputStream os = conn.getOutputStream();
					os.write(data.getBytes());
					os.flush();
				}
			} else {
				conn.setDoOutput(false);
				conn.setRequestProperty("Accept-Encoding", "application/json");
				conn.setRequestProperty("Accept", "application/json");
			}

			responseCode = conn.getResponseCode();
					
			if (conn.getHeaderFields()!=null)
			{
					results = conn.getHeaderFields().get("Result");
					
					if ((results!=null) && (results.size()> 0))
					{
						response.setHeader("Result", (conn.getHeaderFields().get("Result")).get(0));
					}
					
			}	
			
			response.setStatus(responseCode);
			
			
			switch (responseCode) {
			case HttpURLConnection.HTTP_NO_CONTENT: {

				LOGGER.debug("Response code & message");
				LOGGER.debug("Response Code = {}", Integer.toString((responseCode)));
				LOGGER.debug("Response Message = {}", conn.getResponseMessage());
 
				break;
			}
			case HttpURLConnection.HTTP_OK: {

				LOGGER.debug("Your {} request was successfully answered", urlMethod);

				br = new BufferedReader(new InputStreamReader((conn.getInputStream())));

				message = IOUtils.toString(br);

				// Return the JSON formatted data
				response.getWriter().write(message);
				
				break; // optional
			}
				// You can have any number of case statements.
			default: {

				LOGGER.error("Error during ARC service communication.");

				LOGGER.error("Response Code = {}", Integer.toString(responseCode));
				LOGGER.error("Response Message = {}", conn.getResponseMessage());

				LOGGER.error("Error during ARC service communication.");

				br = new BufferedReader(new InputStreamReader((conn.getInputStream())));
				message = IOUtils.toString(br);
				response.getWriter().write(message);
				
			}
			}

		} catch (Exception e) {
			
			if (responseCode == 0) 	response.setStatus(HttpURLConnection.HTTP_INTERNAL_ERROR);
			
			LOGGER.error("Error during ARC service communication check all the proxy configurations in /system/console/configMgr.");
			LOGGER.error("HTTP request failed!", e);
		}
	}

	/**
	 * Utility method for body extract from the POST request call
	 * 
	 * @param request
	 *            Request made to this servlet.
	 * @return The String representation of the request body payload.
	 * @throws IOException
	 *             In case of fail will throw an IOexception.
	 */
	private String extractPostRequestBody(SlingHttpServletRequest request) throws IOException {
		if ("POST".equalsIgnoreCase(request.getMethod())) { 
			Scanner s = new Scanner(request.getInputStream(), "UTF-8").useDelimiter("\\A");
			return s.hasNext() ? s.next() : "";
		}
		return "";
	}

}
