package com.fadel.aem.ExportTool;

import java.io.IOException;
import java.rmi.ServerException;
import java.util.HashMap;
import java.util.Map;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.apache.felix.scr.annotations.Reference;
import org.apache.felix.scr.annotations.sling.SlingServlet;
import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.SlingHttpServletResponse;
import javax.jcr.Session;
import org.apache.sling.commons.json.JSONArray;
import org.apache.sling.commons.json.JSONObject;
//Sling Imports
import org.apache.sling.api.resource.ResourceResolverFactory;
import org.apache.sling.api.resource.ResourceResolver;
import org.apache.sling.api.resource.Resource;
//QueryBuilder APIs
import com.day.cq.search.QueryBuilder;
import com.day.cq.search.Query;
import com.day.cq.search.PredicateGroup;
import com.day.cq.search.result.SearchResult;
import com.fadel.aem.ARCProxy.ARCConfig;
import com.day.cq.search.result.Hit;

//DAM API
import com.day.cq.dam.api.Asset;

@SlingServlet(paths = "/bin/exportExtID", methods = "GET")
public class ExportServlet extends org.apache.sling.api.servlets.SlingAllMethodsServlet {

	
	/** 
	 * Default log. 
	 * */
	protected final Logger log = LoggerFactory.getLogger(this.getClass());

	private Session session;

	/**
	 * OSGI service reference for OSGI config retrieval 
	 */
	@Reference
	ARCConfig config;
	
	/**
	 * Inject a Sling ResourceResolverFactory
	 */
	@Reference
	private ResourceResolverFactory resolverFactory;
		
	/**
	 * Inject Query builder reference 
	 */
	@Reference
	private QueryBuilder builder;

	private static final long serialVersionUID = -3960692666512058119L;

	@Override
	/**
	 * The GET Method uses the AEM QueryBuilder API to retrieve ExternalIDs of
	 * DAM Assets places them in a Json "result" and returns it
	 * in the HTTP output stream
	 */
	protected void doGet(SlingHttpServletRequest request, SlingHttpServletResponse response)
			throws ServerException, IOException {
		
		try {
			JSONObject result = new JSONObject();
			JSONObject jsonHit = new JSONObject();
			JSONArray resultHitsArray = new JSONArray();
			String searchInPath = request.getParameter("path");
			
			if (searchInPath != null && !searchInPath.isEmpty()) {
				
				// Invoke the adaptTo method to create a Session using the resolverFactory
				ResourceResolver resourceResolver = resolverFactory.getAdministrativeResourceResolver(null);
				session = resourceResolver.adaptTo(Session.class);
				Resource rsPath = resourceResolver.resolve(searchInPath);
				Asset assetPath = rsPath.adaptTo(Asset.class);

				//If the requested is an Asset 
				if (assetPath != null) 
				{		
					String fileName = assetPath.getName();
					String filenNameWithoutExt = fileName;
					filenNameWithoutExt = fileName.substring(0,fileName.lastIndexOf("."));
					
					String exterID = "";
					
					if ((config==null) || (config.getFileNameExtID()==null)) 
							exterID = filenNameWithoutExt;
						else
							exterID = config.getFileNameExtID() == true  ? filenNameWithoutExt : assetPath.getMetadataValue("dam:sha1");
					
					
					if (exterID==null) exterID = "";
					
					jsonHit  = new JSONObject();
					
					jsonHit.put("name", fileName);
					jsonHit.put("externalID", exterID);
					jsonHit.put("path", assetPath.getPath());
					resultHitsArray.put(jsonHit);
					
				} 
				//else we assume that is a Folder and we try to search Asset inside it
				else 
				{
					Map<String, String> map = new HashMap<String, String>();

					// set QueryBuilder search criteria
					map.put("type", "dam:Asset");
					map.put("path", searchInPath);
					map.put("property.depth", "10");
					map.put("p.hits", "selective");
					map.put("p.properties", "jcr:path jcr:content/cq:name jcr:content/metadata/dam:sha1");
					map.put("p.limit", "-1");

					builder = resourceResolver.adaptTo(QueryBuilder.class);

					// INvoke the Search query
					Query query = builder.createQuery(PredicateGroup.create(map), session);

					SearchResult sr = query.getResult();

					//Read each hit and adapt them as Assets for properties retrieval 
					for (Hit hit : sr.getHits()) {

						String path = hit.getPath();
						Resource rs = resourceResolver.getResource(path);
						Asset asset = rs.adaptTo(Asset.class);
						
						String sha1 = asset.getMetadataValue("dam:sha1");
						String fileName = asset.getName();
						String filenNameWithoutExt = fileName;
						filenNameWithoutExt = fileName.substring(0,fileName.lastIndexOf("."));
															
						String exterID = "";
						
						if ((config==null) || (config.getFileNameExtID()==null)) 
							exterID = filenNameWithoutExt;
						else
							exterID = config.getFileNameExtID() == true  ? filenNameWithoutExt : sha1;
																	
						if (exterID==null) exterID = "";
						
						jsonHit  = new JSONObject();
						jsonHit.put("name", fileName);
						jsonHit.put("externalID", exterID);
						jsonHit.put("path", path);
						resultHitsArray.put(jsonHit);
					}

				}
			}
			//Write the end response to the main stream
			result.put("result", resultHitsArray);
			response.setContentType("application/json");
			response.setHeader("Content-Type", "application/json");
			response.getWriter().print(result.toString());

		} catch (Exception e) {
			log.error("Export external ID service failed!",e);
		}
	}

}